<?php

namespace Litecms\Subcriber\Http\Controllers;

use App\Http\Controllers\ResourceController as BaseController;
use Form;
use Litecms\Subcriber\Http\Requests\SubscribeRequest;
use Litecms\Subcriber\Interfaces\SubscribeRepositoryInterface;
use Litecms\Subcriber\Models\Subscribe;

/**
 * Resource controller class for subscribe.
 */
class SubscribeResourceController extends BaseController
{

    /**
     * Initialize subscribe resource controller.
     *
     * @param type SubscribeRepositoryInterface $subscribe
     *
     * @return null
     */
    public function __construct(SubscribeRepositoryInterface $subscribe)
    {
        parent::__construct();
        $this->repository = $subscribe;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Litecms\Subcriber\Repositories\Criteria\SubscribeResourceCriteria::class);
    }

    /**
     * Display a list of subscribe.
     *
     * @return Response
     */
    public function index(SubscribeRequest $request)
    {
        $view = $this->response->theme->listView();

        if ($this->response->typeIs('json')) {
            $function = camel_case('get-' . $view);
            return $this->repository
                ->setPresenter(\Litecms\Subcriber\Repositories\Presenter\SubscribePresenter::class)
                ->$function();
        }

        $subscribes = $this->repository->paginate();

        return $this->response->title(trans('subcriber::subscribe.names'))
            ->view('subcriber::subscribe.index', true)
            ->data(compact('subscribes'))
            ->output();
    }

    /**
     * Display subscribe.
     *
     * @param Request $request
     * @param Model   $subscribe
     *
     * @return Response
     */
    public function show(SubscribeRequest $request, Subscribe $subscribe)
    {

        if ($subscribe->exists) {
            $view = 'subcriber::subscribe.show';
        } else {
            $view = 'subcriber::subscribe.new';
        }

        return $this->response->title(trans('app.view') . ' ' . trans('subcriber::subscribe.name'))
            ->data(compact('subscribe'))
            ->view($view, true)
            ->output();
    }

    /**
     * Show the form for creating a new subscribe.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(SubscribeRequest $request)
    {

        $subscribe = $this->repository->newInstance([]);
        return $this->response->title(trans('app.new') . ' ' . trans('subcriber::subscribe.name')) 
            ->view('subcriber::subscribe.create', true) 
            ->data(compact('subscribe'))
            ->output();
    }

    /**
     * Create new subscribe.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(SubscribeRequest $request)
    {
        try {
            $attributes              = $request->all();
            $attributes['user_id']   = user_id();
            $attributes['user_type'] = user_type();
            $subscribe                 = $this->repository->create($attributes);

            return $this->response->message(trans('messages.success.created', ['Module' => trans('subcriber::subscribe.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('subcriber/subscribe/' . $subscribe->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/subcriber/subscribe'))
                ->redirect();
        }

    }

    /**
     * Show subscribe for editing.
     *
     * @param Request $request
     * @param Model   $subscribe
     *
     * @return Response
     */
    public function edit(SubscribeRequest $request, Subscribe $subscribe)
    {
        return $this->response->title(trans('app.edit') . ' ' . trans('subcriber::subscribe.name'))
            ->view('subcriber::subscribe.edit', true)
            ->data(compact('subscribe'))
            ->output();
    }

    /**
     * Update the subscribe.
     *
     * @param Request $request
     * @param Model   $subscribe
     *
     * @return Response
     */
    public function update(SubscribeRequest $request, Subscribe $subscribe)
    {
        try {
            $attributes = $request->all();

            $subscribe->update($attributes);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('subcriber::subscribe.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('subcriber/subscribe/' . $subscribe->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('subcriber/subscribe/' . $subscribe->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the subscribe.
     *
     * @param Model   $subscribe
     *
     * @return Response
     */
    public function destroy(SubscribeRequest $request, Subscribe $subscribe)
    {
        try {

            $subscribe->delete();
            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('subcriber::subscribe.name')]))
                ->code(202)
                ->status('success')
                ->url(guard_url('subcriber/subscribe/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('subcriber/subscribe/' . $subscribe->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove multiple subscribe.
     *
     * @param Model   $subscribe
     *
     * @return Response
     */
    public function delete(SubscribeRequest $request, $type)
    {
        try {
            $ids = hashids_decode($request->input('ids'));

            if ($type == 'purge') {
                $this->repository->purge($ids);
            } else {
                $this->repository->delete($ids);
            }

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('subcriber::subscribe.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('subcriber/subscribe'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/subcriber/subscribe'))
                ->redirect();
        }

    }

    /**
     * Restore deleted subscribes.
     *
     * @param Model   $subscribe
     *
     * @return Response
     */
    public function restore(SubscribeRequest $request)
    {
        try {
            $ids = hashids_decode($request->input('ids'));
            $this->repository->restore($ids);

            return $this->response->message(trans('messages.success.restore', ['Module' => trans('subcriber::subscribe.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('/subcriber/subscribe'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/subcriber/subscribe/'))
                ->redirect();
        }

    }

}
